@@ -674,7 +674,7 @@ class Consumer(object):
         except simplejson.JSONDecodeError as error:
             LOGGER.error('Could not decode message body: %s', error,
                          exc_info=sys.exc_info())
-            return {}
+            raise MessageException(error)
 
     def _load_pickle_value(self, value):
         """Deserialize a pickle string returning the native Python data type
